#!/bin/bash
#

# build_pandora.sh <product_directory> <e5|e6|nu:e5|nu:e6> <debug|opt|prof> [tar]

# if this build script complains that it cannot find bzlib.h, 
# you need to install bzip2-devel on Scientific Linux

usage()
{
   echo "USAGE: `basename ${0}` <product_directory> <e5|e6|nu:e5|nu:e6> <debug|opt|prof> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
}

set_qualifiers()
{
 
    if [ -z ${basequal} ]
    then
       echo "ERROR: please specify the qualifiers"
       usage
       exit 1
    fi

    fullqual=${basequal}:${extraqual}
    qualdir=`echo ${basequal} | sed -e 's/:/-/g'`-${extraqual}
    if [ "${extraqual}" = "opt" ]
    then
	qualver=`echo ${basequal} | sed -e 's/:/_/g'`_${extraqual}
	DOP="BUILD_OPT=1"
    elif [ "${extraqual}" = "debug" ]
    then
	qualver=${extraqual}_`echo ${basequal} | sed -e 's/:/_/g'`
	DOP="BUILD_DEBUG=1"
    elif [ "${extraqual}" = "prof" ]
    then
	qualver=`echo ${basequal} | sed -e 's/:/_/g'`_${extraqual}
	DOP="BUILD_PROF=1"
    else
       echo "ERROR: please specify debug, opt, or prof"
       usage
       exit 1
    fi
    qlist=`echo ${fullqual} | sed -e 's/:/ /g' | tr " " "\n"| sort |tr "\n" " "`
    qualver=`echo ${qlist} | tr " " "_"`

    if [ "${basequal}" = "e5" ] || [ "${basequal}" = "nu:e5" ]
    then
	STDTYPE="BUILD_CXX11=1"
    elif [ "${basequal}" = "e6" ] || [ "${basequal}" = "nu:e6" ]
    then
	STDTYPE="BUILD_CXX1Y=1"
    else
       echo "ERROR: unrecognized qualifier ${basequal}"
       usage
       exit 1
    fi
}

get_platform_name()
{
    OS1=`uname`
    flvr=`ups flavor`
    if [ ${OS1} = "Darwin" ]
    then
	flvr=`ups flavor -2`
    fi
    OS=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform`

    if [ -z ${flvr} ]
    then
       echo "ERROR: could not determine flavor for this platform"
       echo "       something is very wrong"
       exit 1
    fi

    pkgdir=${product_dir}/${package}/${pkgver}/${flvr}-${qualdir}
    patchdir=${product_dir}/${package}/${pkgver}/patch
    blddir=${product_dir}/${package}/${pkgver}/build/${flvr}-${qualdir}
    tardir=${product_dir}/${package}/${pkgver}/tar
    sourcedir=${product_dir}/${package}/${pkgver}/source
}

die()
{
  echo "$1"; exit 2
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=pandora
origpkgver=v00_16
pkgver=${origpkgver}b
ssibuildshims_version=v0_07_04
pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pkgtarfile=Pandora-${origpkgver}.tar.bz2
pndver=`echo ${origpkgver} | sed -e 's/_/-/g'`

get_this_dir

get_ssibuildshims

set_qualifiers

get_platform_name

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

mkdir -p ${pkgdir}
if [ ! -d ${pkgdir} ]
then
   echo "ERROR: failed to create ${pkgdir}"
   exit 1
fi

# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

setup ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${PRODUCTS} || die "ERROR: setup ${package} ${pkgver} failed."

mkdir -p ${PANDORA_FQ_DIR} || die "ERROR: failed to create ${PANDORA_FQ_DIR}"

export PANDORA_RELEASE=${pndver}

cd ${PANDORA_FQ_DIR}
tar xf ${PANDORA_DIR}/tar/Pandora-${PANDORA_RELEASE}.tar.bz2 || die "ERROR: failed to unwind ${PANDORA_DIR}/tar/Pandora-${PANDORA_RELEASE}.tar.bz2"
patch -b -p0 < ${PANDORA_DIR}/patch/${PANDORA_RELEASE}.patch || die "ERROR: patch failed"


set -x
make ${PANDORA_J:+-j ${PANDORA_J}} ${DOP} ${STDTYPE} MONITORING=1 PANDORA_SOURCE=${PANDORA_FQ_DIR} || die "ERROR:make failed"
set +x

if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: build failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi

# real ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup -B ${package} ${pkgver} -q ${fullqual} -z ${product_dir}:${PRODUCTS} || die "ERROR: setup failed."
echo "${package} is installed at ${PANDORA_FQ_DIR}"

# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
fi

exit 0
